<?php
namespace ElementorMinimalPostsRevolution\Widgets;

use \Elementor\Controls_Manager as Controls_Manager;
use \Elementor\Frontend;
use \Elementor\Group_Control_Border as Group_Control_Border;
use \Elementor\Group_Control_Box_Shadow as Group_Control_Box_Shadow;
use \Elementor\Group_Control_Typography as Group_Control_Typography;
use \Elementor\Utils as Utils;
use \Elementor\Widget_Base as Widget_Base;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Elementor Carousel
 *
 * Elementor widget for carousel
 *
 * @since 1.0.0
 */
class Minimal_Carousel extends Widget_Base {

	/**
	 * Retrieve the widget name.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'emp-mega-carousel';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return esc_html__( 'Carousel', 'minimalpostsrevolutionelementor' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'eicon-media-carousel';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'minimal-posts-revolution' ];
	}


	/**
	 * Get post type categories.
	 */
	private function grid_get_all_post_type_categories( $post_type ) {
		$options = array();

		if ( $post_type == 'post' ) {
			$taxonomy = 'category';
		} else {
			$taxonomy = $post_type;
		}

		if ( ! empty( $taxonomy ) ) {
			// Get categories for post type.
			$terms = get_terms(
				array(
					'taxonomy'   => $taxonomy,
					'hide_empty' => false,
				)
			);
			if ( ! empty( $terms ) ) {
				foreach ( $terms as $term ) {
					if ( isset( $term ) ) {
						if ( isset( $term->slug ) && isset( $term->name ) ) {
							$options[ $term->slug ] = $term->name;
						}
					}
				}
			}
		}

		return $options;
	}


	/**
	 * Get post type categories.
	 */
	private function grid_get_all_custom_post_types() {
		$options = array();

		$args = array( '_builtin' => false );
		$post_types = get_post_types( $args, 'objects' );

		foreach ( $post_types as $post_type ) {
			if ( isset( $post_type ) ) {
					$options[ $post_type->name ] = $post_type->label;
			}
		}

		return $options;
	}



	/**
	 * Retrieve the list of scripts the widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget scripts dependencies.
	 */
	public function get_script_depends() {
		return [ 'minimalpostsrevolutionelementor' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function _register_controls() {
		$this->start_controls_section(
			'section_content',
			[
				'label' => esc_html__( 'Content', 'minimalpostsrevolutionelementor' ),
			]
		);

		$this->add_control(
		  'vcmp_carousel_type',
		  [
		    'label' => esc_html__( 'Post Display Type', 'minimalpostsrevolutionelementor' ),
		    'type' => Controls_Manager::SELECT,
		    'default' => 'type1',
		    'options' => [
					'type1' => 'Type 1',
					'type2' => 'Type 2'
		    ]
		  ]
		);

		$this->add_control(
		  'vcmp_carousel_excerpt',
		  [
		    'label' => esc_html__( 'Show Excerpt', 'minimalpostsrevolutionelementor' ),
		    'type' => Controls_Manager::SELECT,
		    'default' => 'true',
		    'options' => [
		      'true' 		=> esc_html__( 'Show', 'minimalpostsrevolutionelementor' ),
		      'false' 	=> esc_html__( 'Hidden', 'minimalpostsrevolutionelementor' )
		    ]
		  ]
		);


		$this->add_control(
		  'vcmp_carousel_excerpt_number',
		  [
		    'label' => esc_html__( 'Number Experpt', 'minimalpostsrevolutionelementor' ),
		    'type' => Controls_Manager::TEXT,
		    'default' => '150',
		    'condition'	=> [
		      'vcmp_carousel_excerpt'	=> 'true'
		    ]
		  ]
		);

		$this->add_control(
		  'vcmp_carousel_date_format',
		  [
		    'label' => esc_html__( 'Date Format', 'minimalpostsrevolutionelementor' ),
		    'type' => Controls_Manager::SELECT,
		    'default' => 'F j, Y',
		    'options' => [
		        'F j, Y g:i a'			=> esc_html__( 'November 6, 2010 12:50 am','minimalpostsrevolutionelementor'),
		      'F j, Y'				=> esc_html__( 'November 6, 2010','minimalpostsrevolutionelementor'),
		        'F, Y'					=> esc_html__( 'November, 2010','minimalpostsrevolutionelementor'),
		      'g:i a'					=> esc_html__( '12:50 am','minimalpostsrevolutionelementor'),
		        'g:i:s a'				=> esc_html__( '12:50:48 am','minimalpostsrevolutionelementor'),
		      'l, F jS, Y'			=> esc_html__( 'Saturday, November 6th, 2010','minimalpostsrevolutionelementor'),
		        'M j, Y @ G:i'			=> esc_html__( 'Nov 6, 2010 @ 0:50','minimalpostsrevolutionelementor'),
		      'Y/m/d \a\t g:i A'		=> esc_html__( '2010/11/06 at 12:50 AM','minimalpostsrevolutionelementor'),
		        'Y/m/d \a\t g:ia'		=> esc_html__( '2010/11/06 at 12:50am','minimalpostsrevolutionelementor'),
		      'Y/m/d g:i:s A'			=> esc_html__( '2010/11/06 12:50:48 AM','minimalpostsrevolutionelementor'),
		      'Y/m/d'					=> esc_html__( '2010/11/06','minimalpostsrevolutionelementor')
		    ],
		  ]
		);




		$this->end_controls_section();

  		$this->start_controls_section(
  			'section_carousel_settings',
  			[
  				'label' => esc_html__( 'Carousel Settings', 'essential-addons-elementor' )
  			]
		);

		$this->add_control(
			'vcmp_carousel_item',
			[
				'label' => esc_html__( 'Number Item Show', 'elementor-fast-gallery' ),
				'type' => Controls_Manager::SELECT,
				'default' => '3',
				'options' => [
					'1' => '1',
					'2' => '2',
					'3' => '3'
				]
			]
		);

		$this->add_control(
			'vcmp_carousel_navigation',
			[
				'label' => esc_html__( 'Navigation', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'true',
				'options' => [
					'true' 		=> esc_html__( 'Show', 'minimalpostsrevolutionelementor' ),
					'false' 	=> esc_html__( 'Hidden', 'minimalpostsrevolutionelementor' )
				]
			]
		);

		$this->add_control(
			'vcmp_carousel_pagination',
			[
				'label' => esc_html__( 'Pagination', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'true',
				'options' => [
					'true' 		=> esc_html__( 'Show', 'minimalpostsrevolutionelementor' ),
					'false' 	=> esc_html__( 'Hidden', 'minimalpostsrevolutionelementor' )
				]
			]
		);

		$this->add_control(
			'vcmp_carousel_autoplay',
			[
				'label' => esc_html__( 'Autoplay (ms) - ex 1000 (Insert 0 if you want disable autoplay)', 'minimalpostsrevolutionelementor' ),
				'label_block' => true,
				'type' => Controls_Manager::TEXT,
				'default' => '2000'
			]
		);




		$this->end_controls_section();

  		$this->start_controls_section(
  			'section_query',
  			[
  				'label' => esc_html__( 'QUERY', 'essential-addons-elementor' )
  			]
		);

		$this->add_control(
			'vcmp_query_source',
			[
				'label' => esc_html__( 'Source', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'wp_posts',
				'options' => [
					'wp_posts' 				=> esc_html__( 'Wordpress Posts', 'essential-addons-elementor' ),
					'wp_custom_posts_type' 	=> esc_html__( 'Custom Posts Type', 'essential-addons-elementor' )
				]
			]
		);

		$this->add_control(
			'vcmp_query_sticky_posts',
			[
				'label' => esc_html__( 'All Posts/Sticky posts', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'allposts',
				'options' => [
					'allposts' 			=> esc_html__( 'All Posts', 'essential-addons-elementor' ),
					'onlystickyposts'	=> esc_html__( 'Only Sticky Posts', 'essential-addons-elementor' )
				],
				'condition'	=> [
					'vcmp_query_source'	=> 'wp_posts'
				]
			]
		);

		$this->add_control(
			'vcmp_query_posts_type',
			[
				'label' => esc_html__( 'Select Post Type Source', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'options' => $this->grid_get_all_custom_post_types(),
				'condition'	=> [
					'vcmp_query_source'	=> 'wp_custom_posts_type'
				]
			]
		);

		$this->add_control(
			'vcmp_query_categories',
			[
				'label' => esc_html__( 'Categories', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT2,
				'multiple' => true,
				'options' => $this->grid_get_all_post_type_categories('post'),
				'condition'	=> [
					'vcmp_query_source'	=> 'wp_posts'
				]
			]
		);

		$this->add_control(
			'vcmp_query_order',
			[
				'label' => esc_html__( 'Order', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'DESC',
				'options' => [
					'DESC'	=> 'DESC',
					'ASC' 	=> 'ASC'
				]
			]
		);

		$this->add_control(
			'vcmp_query_orderby',
			[
				'label' => esc_html__( 'Order By', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'date',
				'options' => [
					'date'			=> 'Date',
					'ID' 			=> 'ID',
					'author' 		=> 'Author',
					'title' 		=> 'Title',
					'name' 			=> 'Name',
					'modified'		=> 'Modified',
					'parent' 		=> 'Parent',
					'rand' 			=> 'Rand',
					'comment_count' => 'Comments Count',
					'none' 			=> 'None'
				]
			]
		);

		$this->add_control(
			'vcmp_query_number',
			[
				'label' => esc_html__( 'Number Posts', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::TEXT,
				'default' => '5'
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'section_animation',
			[
				'label' => esc_html__( 'Animations', 'minimalpostsrevolutionelementor' )
			]
		);

		$this->add_control(
			'vcmp_animate',
			[
				'label' => esc_html__( 'Animate', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'off',
				'options' => [
					'off'	=> 'Off',
					'on' 	=> 'On'
				]
			]
		);

		$this->add_control(
			'vcmp_animate_effect',
			[
				'label' => esc_html__( 'Animate Effects', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'fade-in',
				'options' => [
					'fade-in'		=> 'Fade In',
					'fade-in-up' 	=> 'fade in up',
					'fade-in-down' 	=> 'fade in down',
					'fade-in-left' 	=> 'fade in Left',
					'fade-in-right' => 'fade in Right',
					'fade-out'		=> 'Fade In',
					'fade-out-up' 	=> 'Fade Out up',
					'fade-out-down' => 'Fade Out down',
					'fade-out-left' => 'Fade Out Left',
					'fade-out-right' => 'Fade Out Right',
					'bounce-in'			=> 'Bounce In',
					'bounce-in-up' 		=> 'Bounce in up',
					'bounce-in-down' 	=> 'Bounce in down',
					'bounce-in-left' 	=> 'Bounce in Left',
					'bounce-in-right' 	=> 'Bounce in Right',
					'bounce-out'		=> 'Bounce In',
					'bounce-out-up' 	=> 'Bounce Out up',
					'bounce-out-down' 	=> 'Bounce Out down',
					'bounce-out-left' 	=> 'Bounce Out Left',
					'bounce-out-right' 	=> 'Bounce Out Right',
					'zoom-in'			=> 'Zoom In',
					'zoom-in-up' 		=> 'Zoom in up',
					'zoom-in-down' 		=> 'Zoom in down',
					'zoom-in-left' 		=> 'Zoom in Left',
					'zoom-in-right' 	=> 'Zoom in Right',
					'zoom-out'			=> 'Zoom In',
					'zoom-out-up' 		=> 'Zoom Out up',
					'zoom-out-down' 	=> 'Zoom Out down',
					'zoom-out-left' 	=> 'Zoom Out Left',
					'zoom-out-right' 	=> 'Zoom Out Right',
					'flash' 			=> 'Flash',
					'strobe'			=> 'Strobe',
					'shake-x'			=> 'Shake X',
					'shake-y'			=> 'Shake Y',
					'bounce' 			=> 'Bounce',
					'tada'				=> 'Tada',
					'rubber-band'		=> 'Rubber Band',
					'swing' 			=> 'Swing',
					'spin'				=> 'Spin',
					'spin-reverse'		=> 'Spin Reverse',
					'slingshot'			=> 'Slingshot',
					'slingshot-reverse'	=> 'Slingshot Reverse',
					'wobble'			=> 'Wobble',
					'pulse' 			=> 'Pulse',
					'pulsate'			=> 'Pulsate' ,
					'heartbeat'			=> 'Heartbeat',
					'panic' 			=> 'Panic'
				],
				'condition'	=> [
					'vcmp_animate'	=> 'on'
				]
			]
		);

		$this->add_control(
			'vcmp_delay',
			[
				'label' => esc_html__( 'Animate Delay (ms)', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::TEXT,
				'default' => '2000',
				'condition'	=> [
					'vcmp_animate'	=> 'on'
				]
			]
		);

		$this->end_controls_section();


		$this->start_controls_section(
			'section_style',
			[
				'label' => esc_html__( 'Style', 'minimalpostsrevolutionelementor' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'vcmp_custom_style',
			[
				'label' => esc_html__( 'Custom Style', 'minimalpostsrevolutionelementor' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'off',
				'options' => [
					'off'	=> 'Off',
					'on' 	=> 'On'
				]
			]
		);


		$this->add_control(
			'vcmp_main_color',
			[
				'label' => esc_html__( 'Main Color', 'minimalpostsrevolutionelementor' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'default' => '#7FB1C5',
				'condition'	=> [
					'vcmp_custom_style'	=> 'on'
				]
			]
		);

		$this->add_control(
			'vcmp_hover_color',
			[
				'label' => esc_html__( 'Hover Color', 'minimalpostsrevolutionelementor' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'default' => '#7FB1C5',
				'condition'	=> [
					'vcmp_custom_style'	=> 'on'
				]
			]
		);


		$this->add_control(
			'vcmp_background_hover',
			[
				'label' => esc_html__( 'Backgound Hover Color', 'minimalpostsrevolutionelementor' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'default' => '#FC615D',
				'condition'	=> [
					'vcmp_custom_style'	=> 'on'
				]
			]
		);

		$this->end_controls_section();
	}


	 /**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		static $instance = 0;
		$instance++;
		$settings = $this->get_settings_for_display();


				$vcmp_carousel_type 					= esc_html($settings['vcmp_carousel_type']);
				$vcmp_carousel_excerpt 				= esc_html($settings['vcmp_carousel_excerpt']);
				$vcmp_carousel_excerpt_number	= esc_html($settings['vcmp_carousel_excerpt_number']);
				$vcmp_carousel_date_format	 	= esc_html($settings['vcmp_carousel_date_format']);


				$vcmp_carousel_item 			= esc_html($settings['vcmp_carousel_item']);
				$vcmp_carousel_navigation 		= esc_html($settings['vcmp_carousel_navigation']);
				$vcmp_carousel_pagination 		= esc_html($settings['vcmp_carousel_pagination']);
				$vcmp_carousel_autoplay 		= esc_html($settings['vcmp_carousel_autoplay']);
				$vcmp_carousel_item_padding 	= '';

				$vcmp_query_source					= esc_html($settings['vcmp_query_source']);
				$vcmp_query_sticky_posts 			= esc_html($settings['vcmp_query_sticky_posts']);
				$vcmp_query_posts_type 				= esc_html($settings['vcmp_query_posts_type']);
				$vcmp_query_categories = '';
				if(!empty($settings['vcmp_query_categories'])) {
					$num_cat = count($settings['vcmp_query_categories']);
					$i = 1;
					foreach ( $settings['vcmp_query_categories'] as $element ) {
						$vcmp_query_categories .= $element;
						if($i != $num_cat) {
							$vcmp_query_categories .= ',';
						}
						$i++;
					}
				}
				$vcmp_query_order 					= esc_html($settings['vcmp_query_order']);
				$vcmp_query_orderby 				= esc_html($settings['vcmp_query_orderby']);
				$vcmp_query_pagination	 			= '';
				$vcmp_query_pagination_type 			= '';
				$vcmp_query_number	 					= esc_html($settings['vcmp_query_number']);
				$vcmp_query_posts_for_page	 			= '';

				$vcmp_custom_style					= esc_html($settings['vcmp_custom_style']);
				$vcmp_main_color 					= esc_html($settings['vcmp_main_color']);
				$vcmp_hover_color 					= esc_html($settings['vcmp_hover_color']);
				$vcmp_background_hover 					= esc_html($settings['vcmp_background_hover']);


				$vcmp_animate 						= esc_html($settings['vcmp_animate']);
				$vcmp_animate_effect 				= esc_html($settings['vcmp_animate_effect']);
				$vcmp_delay 						= esc_html($settings['vcmp_delay']);

	/* LOAD CSS && JS */
	wp_enqueue_style( 'normalize' );
	wp_enqueue_style( 'animations' );
	wp_enqueue_style( 'fonts-vc' );
	wp_enqueue_style( 'minimal-posts-revolution-posts' );
	wp_enqueue_style( 'minimal-posts-revolution-style' );
	wp_enqueue_script( 'appear' );
	wp_enqueue_script( 'animate' );
	wp_enqueue_style( 'owlcarousel' );
	wp_enqueue_style( 'owltheme' );
	wp_enqueue_script( 'owlcarousel' );

	/* CHECK VALUE EMPTY */
	if($vcmp_carousel_autoplay == '' || $vcmp_carousel_autoplay == '0') { $vcmp_carousel_autoplay = 'false'; }
	if($vcmp_carousel_excerpt_number == '') { $vcmp_carousel_excerpt_number = '50'; }

	$return = '<style type="text/css">';
	if($vcmp_custom_style == 'on') {
		if($vcmp_carousel_type == 'type1') {
			$return .=  '#admp-carousel-'.$instance.' .owl-item .item .admp-info-left {
							background:'.$vcmp_background_hover.'!important;
						}';
		  $return .=  '.minimalpostsrevolutionelementor.posts_carousel_type1.admpselector-'.$instance.' .admp-category a {
													color:'.$vcmp_main_color.'!important;
									}
									.minimalpostsrevolutionelementor.posts_carousel_type1.admpselector-'.$instance.' .admp-category a:hover {
										color:'.$vcmp_hover_color.'!important;
									}
									.minimalpostsrevolutionelementor.posts_carousel_type1.admpselector-'.$instance.' .admp-title a:hover {
										color:'.$vcmp_hover_color.'!important;
									}';
			$return .=  '.owl-theme .owl-controls .owl-page span {
    								border-color:'.$vcmp_main_color.'!important;
									}
									.owl-theme .owl-controls .owl-page.active span, .owl-theme .owl-controls.clickable .owl-page:hover span {
    								background: '.$vcmp_main_color.'!important;
									}
									.owl-theme .owl-controls .owl-buttons div {
										border-color:'.$vcmp_main_color.'!important;
										color:'.$vcmp_main_color.'!important;
									}
									.owl-theme .owl-controls.clickable .owl-buttons div:hover{
										background: '.$vcmp_main_color.'!important;
										color:#ffffff !important;
									}';
		}



		if($vcmp_carousel_type == 'type2') {
			$return .=  '#admp-carousel-'.$instance.' .minimalpostsrevolutionelementor-thumbs-container:hover .admp-info-left {
		 	     		background:'.$vcmp_background_hover.'!important;
			}';
			$return .=  '.minimalpostsrevolutionelementor.posts_carousel_type2.admpselector-'.$instance.' .admp-category a {
													color:'.$vcmp_main_color.'!important;
									}
									.minimalpostsrevolutionelementor.posts_carousel_type2.admpselector-'.$instance.' .admp-category a:hover {
										color:'.$vcmp_hover_color.'!important;
									}
									.minimalpostsrevolutionelementor.posts_carousel_type2.admpselector-'.$instance.' .admp-title a:hover {
										color:'.$vcmp_hover_color.'!important;
									}';
			$return .=  '.owl-theme .owl-controls .owl-page span {
						    	border-color:'.$vcmp_main_color.'!important;
									}
									.owl-theme .owl-controls .owl-page.active span, .owl-theme .owl-controls.clickable .owl-page:hover span {
						    	background: '.$vcmp_main_color.'!important;
									}
									.owl-theme .owl-controls .owl-buttons div {
										border-color:'.$vcmp_main_color.'!important;
										color:'.$vcmp_main_color.'!important;
									}
									.owl-theme .owl-controls.clickable .owl-buttons div:hover{
										background: '.$vcmp_main_color.'!important;
										color:#ffffff !important;
									}';
		}
	}
	$return .= '</style>';

	if($vcmp_carousel_type == 'type3') {

		$return .= '<script type="text/javascript">
					  jQuery(document).ready(function($){
							$("#admp-carousel-'.$instance.'").owlCarousel({
								items : '.$vcmp_carousel_item.',
								navigation: '.$vcmp_carousel_navigation.',
								pagination: '.$vcmp_carousel_pagination.',
								autoPlay:	'.$vcmp_carousel_autoplay.',
								navigationText : ["<i class=\'fa fa-arrow-left\'></i>","<i class=\'fa fa-arrow-right\'></i>"],
							});
						});
					</script>';

	} else {

		$return .= '<script type="text/javascript">
					  jQuery(document).ready(function($){
							$("#admp-carousel-'.$instance.'").owlCarousel({
								items : '.$vcmp_carousel_item.',
								navigation: '.$vcmp_carousel_navigation.',
								pagination: '.$vcmp_carousel_pagination.',
								autoPlay:	'.$vcmp_carousel_autoplay.'
							});
						});
					</script>';

	}


		// LOOP QUERY
		$query = minimalpostsrevolutionelementor_query($vcmp_query_source,
							$vcmp_query_sticky_posts,
							$vcmp_query_posts_type,
							$vcmp_query_categories,
							$vcmp_query_order,
							$vcmp_query_orderby,
							'no',
							$vcmp_query_pagination_type,
							$vcmp_query_number,
							$vcmp_query_posts_for_page);


			$return .= '<div class="adclear"></div>';

		if($vcmp_animate == 'on') { // ANIMATION ON
			$return .= '<div class="animate-in" data-anim-type="'.$vcmp_animate_effect.'" data-anim-delay="'.$vcmp_delay.'">';
		}






/**********************************************************************/
/******************************** TYPE 1 ******************************/
/**********************************************************************/

		if($vcmp_carousel_type == 'type1') {

			$return .= '<div id="admp-carousel-'.$instance.'" class="owl-carousel'.$instance.' minimalpostsrevolutionelementor posts_carousel_type1 admpselector-'.$instance.'">'; // OPEN MAIN DIV

			$count = 0;
			$loop = new \WP_Query($query);
			if($loop) {
			while ( $loop->have_posts() ) : $loop->the_post();
			$link = get_permalink();
			$categories = get_the_category();


			$return .= '<div class="item">';

							$return .= '<div class="minimalpostsrevolutionelementor-thumbs-container">'.minimalpostsrevolutionelementor_thumbs();
							$return .= '</div>';

							$return .= '<div class="admp-info-left">';
									$return .= '<span class="admp-category">'.minimalpostsrevolutionelementor_category($vcmp_query_source,$vcmp_query_posts_type).'</span>';
									$return .= '<span class="admp-title"><a href="'.$link.'">'.get_the_title().'</a></span>';
									if($vcmp_carousel_excerpt == 'true') {
										$return .= '<span class="admp-content">'.minimalpostsrevolutionelementor_excerpt($vcmp_carousel_excerpt_number).'</span>';
									}
									$return .= '<span class="admp-author">'.get_the_author().'</span>';
									$return .= '<span class="admp-date">'.get_the_date($vcmp_carousel_date_format).'</span>';
							$return .= '</div>';



			$return .= '</div>';

			$count++;
			endwhile;
			}

			$return .= '</div>'; // CLOSE MAIN DIV




/**********************************************************************/
/******************************** TYPE 2 ******************************/
/**********************************************************************/

		} elseif($vcmp_carousel_type == 'type2') {

			$return .= '<div id="admp-carousel-'.$instance.'" class="owl-carousel'.$instance.' minimalpostsrevolutionelementor posts_carousel_type2 admpselector-'.$instance.'">'; // OPEN MAIN DIV

			$count = 0;
			$loop = new \WP_Query($query);
			if($loop) {
			while ( $loop->have_posts() ) : $loop->the_post();
			$link = get_permalink();
			$categories = get_the_category();

			$return .= '<div class="item">';

						$return .= '<div class="minimalpostsrevolutionelementor-thumbs-container">'.minimalpostsrevolutionelementor_thumbs();

									$return .= '<div class="admp-info-left">';
											$return .= '<span class="admp-category">'.minimalpostsrevolutionelementor_category($vcmp_query_source,$vcmp_query_posts_type).'</span>';
											$return .= '<span class="admp-title"><a href="'.$link.'">'.get_the_title().'</a></span>';
											if($vcmp_carousel_excerpt == 'true') {
												$return .= '<span class="admp-content">'.minimalpostsrevolutionelementor_excerpt($vcmp_carousel_excerpt_number).'</span>';
											}
											$return .= '<span class="admp-author">'.get_the_author().'</span>';
											$return .= '<span class="admp-date">'.get_the_date($vcmp_carousel_date_format).'</span>';
									$return .= '</div>';


						$return .= '</div>';//img

			$return .= '</div>';//item

			$count++;
			endwhile;
			}

			$return .= '</div>'; // CLOSE MAIN DIV


		}
		if($vcmp_animate == 'on') {

			$return .= '</div>';

		}
		echo $return;
	}

	/**
	 * Render the widget output in the editor.
	 *
	 * Written as a Backbone JavaScript template and used to generate the live preview.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function _content_template() {}
}
